@@ -31,6 +31,18 @@ sub import
     eval { Alzabo::Runtime::Schema->load_from_file( name => $_ ); } foreach @_;
 }
 
+sub sqlmaker
+{
+    my ($schema, $p) = @_;
+
+    my %sqlmaker_p = ( exists $p->{quote_identifiers} ?
+                       ( quote_identifiers => $p->{quote_identifiers} ) :
+                       ()
+                     );
+
+    return $schema->sqlmaker(%sqlmaker_p);
+}
+
 sub process_where_clause
 {
     my ($sql, $where) = @_;
@@ -77,51 +89,51 @@ sub _process_conditions
     my $x = 0;
     foreach my $clause (@$conditions)
     {
-	if (ref $clause)
-	{
-	    Alzabo::Exception::Params->throw
+        if (ref $clause)
+        {
+            Alzabo::Exception::Params->throw
                 ( error => "Individual where clause components must be array references" )
                     unless UNIVERSAL::isa( $clause, 'ARRAY' );
 
-	    Alzabo::Exception::Params->throw
+            Alzabo::Exception::Params->throw
                 ( error => "Individual where clause components cannot be empty" )
                     unless @$clause;
 
-	    if ($needs_op)
-	    {
-		my $op = $x || $has_start ? 'and' : $needed_op;
-		$sql->$op();
-	    }
+            if ($needs_op)
+            {
+                my $op = $x || $has_start ? 'and' : $needed_op;
+                $sql->$op();
+            }
 
-	    $sql->condition(@$clause);
-	    $needs_op = 1;
-	}
-	elsif (lc $clause eq 'and' || lc $clause eq 'or')
-	{
-	    $sql->$clause();
-	    $needs_op = 0;
-	    next;
-	}
-	elsif ($clause eq '(')
-	{
-	    if ($needs_op)
-	    {
-		my $op = $x || $has_start ? 'and' : $needed_op;
-		$sql->$op();
-	    }
-	    $sql->subgroup_start;
-	    $needs_op = 0;
-	}
-	elsif ($clause eq ')')
-	{
-	    $sql->subgroup_end;
-	    $needs_op = 1;
-	}
-	else
-	{
-	    Alzabo::Exception::Params->throw( error => "Invalid where clause specification: $clause" );
-	}
-	$x++;
+            $sql->condition(@$clause);
+            $needs_op = 1;
+        }
+        elsif (lc $clause eq 'and' || lc $clause eq 'or')
+        {
+            $sql->$clause();
+            $needs_op = 0;
+            next;
+        }
+        elsif ($clause eq '(')
+        {
+            if ($needs_op)
+            {
+                my $op = $x || $has_start ? 'and' : $needed_op;
+                $sql->$op();
+            }
+            $sql->subgroup_start;
+            $needs_op = 0;
+        }
+        elsif ($clause eq ')')
+        {
+            $sql->subgroup_end;
+            $needs_op = 1;
+        }
+        else
+        {
+            Alzabo::Exception::Params->throw( error => "Invalid where clause specification: $clause" );
+        }
+        $x++;
     }
 
     $sql->subgroup_end if $has_start;
@@ -144,22 +156,22 @@ sub _process_by_clause
     my @items;
     if ( UNIVERSAL::isa( $by, 'Alzabo::Column' ) || UNIVERSAL::isa( $by, 'Alzabo::SQLMaker::Function' ) )
     {
-	@items = $by;
+        @items = $by;
     }
     elsif ( UNIVERSAL::isa( $by, 'ARRAY' ) )
     {
-	@items = @$by;
+        @items = @$by;
     }
     else
     {
-	Alzabo::Exception::Params->throw( error => "No columns provided for order by" )
-		unless $by->{columns};
+        Alzabo::Exception::Params->throw( error => "No columns provided for order by" )
+                unless $by->{columns};
 
-	push @items, ( UNIVERSAL::isa( $by->{columns}, 'ARRAY' ) ?
-		       @{ $by->{columns} } :
-		       $by->{columns} );
+        push @items, ( UNIVERSAL::isa( $by->{columns}, 'ARRAY' ) ?
+                       @{ $by->{columns} } :
+                       $by->{columns} );
 
-	push @items, lc $by->{sort} if exists $by->{sort};
+        push @items, lc $by->{sort} if exists $by->{sort};
     }
 
     my $method = "${type}_by";
@@ -205,4 +217,3 @@ failure later on.
 Dave Rolsky, <autarch@urth.org>
 
 =cut
-
